/*
  ==================================================================================================
  ___  ___   _   ___ ___  
 | _ )/ _ \ /_\ | _ \   \ 
 | _ \ (_) / _ \|   / |) |
 |___/\___/_/ \_\_|_\___/ 
                          
  DialogCtrlBoardBoard gathers BoardState data and maintains the current board view   
  ==================================================================================================
  Board Controller
    messageIndex - the index of the message that the view is currently looking at
*/
var DialogCtrlBoard = function() {
  'use strict';
  
  //  Determines at what point to switch to fetching intial messages from AWS 
  //    rather than fetching from sidebar (necessary because fetching initial messages 
  //    from the sidebar is slower than getting it from AWS when you have a lot of messages)
  var MESSAGE_TRANSFER_THRESHOLD = 2000;

  var self;
  var inkPathsHandler;
  var ui;
  var nbInterface;
  var kappInterface;
  var dialogCtrl;
  
  function DialogCtrlBoard(pInkPathsHandler, pUi, pNbInterface, pKappInterface, pDialogCtrl) {
    self = this;
    inkPathsHandler = pInkPathsHandler;
    ui = pUi;
    nbInterface = pNbInterface;
    kappInterface = pKappInterface;
    dialogCtrl = pDialogCtrl;
    this.messageIndex = 0; //the current message being displayed up to
  }

/*
  _   _ ___ ___   _ _____ ___   ___ _   _ _  _  ___ _____ ___ ___  _  _ ___ 
 | | | | _ \   \ /_\_   _| __| | __| | | | \| |/ __|_   _|_ _/ _ \| \| / __|
 | |_| |  _/ |) / _ \| | | _|  | _|| |_| | .` | (__  | |  | | (_) | .` \__ \
  \___/|_| |___/_/ \_\_| |___| |_|  \___/|_|\_|\___| |_| |___\___/|_|\_|___/
                                                                            
*/
  // Called whenever the canvas is updated and when the add-on disconnects
  DialogCtrlBoard.prototype.updateLive = function() {
    var isLive = this.messageIndex == BoardState.messageCount && BoardState.status === "connected";
    if (!dialogCtrl.crop.isCropping) {
      ui.status.toggleLive(isLive);
      ui.toggleColorButtons(isLive);
    }
  };

  // Updates the board state and performs other updates
  DialogCtrlBoard.prototype.updateState = function(obj) {
    var prevMessageCount = BoardState === false ? 0 : BoardState.messageCount;
    var prevMessages = BoardState.messages;
    var newState = obj.eventData;
    var newStateMessagesLen = newState.messages.length;
    for (var i = 0; i < newStateMessagesLen; i++) {
      BoardState.messages[BoardState.messageCount++] = newState.messages[i];
    }
    BoardState.keyframes = newState.keyframes;
    BoardState.snapshots = newState.snapshots;
    BoardState.keyframeCount = newState.keyframeCount;
    BoardState.snapshotCount = newState.snapshotCount;
    BoardState.status = newState.status;
    self.updateColors(prevMessageCount);
    dialogCtrl.scrubber.updateSlider();
    var messageCount = BoardState.messageCount;
    if (self.messageIndex == prevMessageCount) {
      dialogCtrl.scrubber.setScrubTime(messageCount);
    }
    if (messageCount > 0 && !dialogCtrl.ink.isDrawing) {
      ui.status.toggleMessage(false);
    }
  };

  // Replays all of the old messages for the snapshot on the canvas
  // Relies on the board state and slider to be up to date
  // Gets called when the slider changes and the event is triggered
  // i.e. as a sideeffect of DialogCtrlBoard.scrubber.setScrubTime
  DialogCtrlBoard.prototype.updateCanvas = function(startVal, index) {
    var messages = BoardState.messages;
    var canvas = ui.elems.$canvas[0];
    var context = canvas.getContext('2d');
    var width = BoardState.session.boardWidth;
    var scale = ui.elems.$canvas.width() / width;
    var height = BoardState.session.boardHeight;
    if (startVal === 0) {
      canvas.setAttribute('width', width * scale);
      canvas.setAttribute('height', height * scale);
      context.fillStyle = 'white';
      context.rect(0, 0, width, height);
      context.fill();
    }
    // Get all the strokes and draw them on the temporary canvas
    for (var i = startVal; i < index; i++) {
      if (messages[i] !== undefined) {
        // Draw the stroke
        var currColor = messages[i].color ? messages[i].color : {R:0, G:0, B:0};
        inkPathsHandler.drawScaledPathsBezier(messages[i], context, scale, currColor);        
      }
    }
    this.updateLive();
  };

  // Tells the add-on there's been a color change in the messages
  DialogCtrlBoard.prototype.updateColors = function(oldLength) {
    var data = {};
    data.color = {};
    data.color.R = dialogCtrl.ink.color.R;
    data.color.G = dialogCtrl.ink.color.G;
    data.color.B = dialogCtrl.ink.color.B;
    data.indices = [];
    for (var i = oldLength; i < BoardState.messageCount; i++) {
      if (BoardState.messages[i] !== undefined && !BoardState.messages[i].color && BoardState.messages[i].type != BoardState.constants.type_erase) {
        data.indices.push(i);
        BoardState.messages[i].color = data.color;
      }
    }
    if (data.indices.length > 0) {
      nbInterface.postEvent("updateColors", data);
    }
  };

  // Updates the canvas with a board clear
  DialogCtrlBoard.prototype.clearBoard = function(obj) {
    ui.elems.$blackInk[0].click();
    self.initState(obj);
  };

  // Registers that the board is no longer live
  DialogCtrlBoard.prototype.disconnected = function() {
    BoardState.status = "disconnected";
    self.updateLive();
  };

/*
  ___ _  _ ___ _____ ___   _   _    ___ ____  _ _____ ___ ___  _  _ 
 |_ _| \| |_ _|_   _|_ _| /_\ | |  |_ _|_  / /_\_   _|_ _/ _ \| \| |
  | || .` || |  | |  | | / _ \| |__ | | / / / _ \| |  | | (_) | .` |
 |___|_|\_|___| |_| |___/_/ \_\____|___/___/_/ \_\_| |___\___/|_|\_|
                                                                    
*/

  // Tell the add-on we're done initializing
  
  DialogCtrlBoard.prototype.sendWave = function() {
    nbInterface.postEvent("wave");
  };

  
  DialogCtrlBoard.prototype.sendLoadedWave = function() {
    var data = {"id": BoardState.messageCount};
    nbInterface.postEvent("loadedWave", data);
  };

  
  // Takes the messages color data from the BoardState's colorData field 
  // and adds it to the individual messages in the BoardState's colors field.
  // This is needed so that the drawing code can see the color of each message,
  // since the drawing code looks at each message's message.color field rather
  // than the BoardState's BoardState.colorData field
  DialogCtrlBoard.prototype.initMessageColors = function() {
    var colors = BoardState.colorData;
    var colorsLen = colors.length;
    for (var i=0; i<colorsLen; i++) {
      var color = colors[i];
      if (color && BoardState.messages[i]) {
        BoardState.messages[i].color = color;
      }
    }
  };

  // Called if the normal init is unable to retrieve messages from the server
  // Like if the session has been disconnected
  DialogCtrlBoard.prototype.fallbackInit = function(obj) {
    BoardState = obj.eventData;
    self.initMessageColors();
    dialogCtrl.scrubber.initSlider();
    dialogCtrl.scrubber.playing = false;
    dialogCtrl.scrubber.setScrubTime(BoardState.messageCount);
    dialogCtrl.crop.resetCropper();
    ui.status.toggleMessage(false);
    if (BoardState.messageCount === 0) {
      ui.status.toggleMessage(true, ui.MESSAGE_EMPTY, 50);
    }
    self.sendLoadedWave();
  };  

  // Called upon to initialize state after loading session
  DialogCtrlBoard.prototype.initState = function(obj) {
    BoardState = obj.eventData;
    BoardState.messages = [];
    var expectedMessageCount = BoardState.messageCount;
    BoardState.messageCount = 0;
    BoardState.session.sequenceId = 0;
    ui.initWindow();
    // You can comment out getMessages() and replace it with requestFallbackData() if you would
    // rather it grab data from the sidebar instead of the server - faster with small sessions
    // and/or slow internet connections
    // Currently set to only run getMessages again if there are more than 2000 messages in the
    // initial retrieval. In this case, even on bad internet connections, grabbing the messages
    // again is considerably faster.
    if (expectedMessageCount > MESSAGE_TRANSFER_THRESHOLD) {
      var onSuccessCallback = function() {
        printDebug("Messages received.");
        nbInterface.parseMessages(data);
        self.initMessageColors();
        dialogCtrl.initSlider();
        dialogCtrl.scrubber.playing = true;
        dialogCtrl.scrubber.setScrubTime(BoardState.messageCount);
        dialogCtrl.crop.resetCropper();
        ui.status.toggleMessage(false);
        if (BoardState.messageCount === 0) {
          ui.status.toggleMessage(true, ui.MESSAGE_EMPTY);
        }
        self.sendLoadedWave();
      };
      var onErrorCallback = function() {
        self.requestFallbackData();
      };
      kappInterface.getMessages(onSuccessCallback, onErrorCallback);
    } else {
      nbInterface.postEvent("getMessagesFailed"); //request fallback data
    }
  };

  return DialogCtrlBoard;
}();