/* global printDebug */
// Handles the sending of analytic data to Google Analytics, and tracks how ink is used
// once inserted into the Notebook page.

var Analytics = function() {

	var that;

	var inkOnPageTrackers;
	var groupSet;

	function Analytics() {
		that = this;
		inkOnPageTrackers = [];
		groupSet = {};
		that.enableInkOnPageTracking();
	}

	// Send a tracked event to Google Analytics
	Analytics.prototype.sendTrackedEvent = function(category, action, label, value) {
		ga('send', 'event', category, action, label, value);
		printDebug("Analytics: " + category + " - " + action + " - " + label + " - " + value);
	};

	// Adds a group of ink to be tracked. Takes the guid of the group.
	Analytics.prototype.addInkGroupToTracking = function(groupId) {
		groupSet[groupId] = true;
	};

	// Enables observers to track how ink is used after being inserted to the page
	Analytics.prototype.enableInkOnPageTracking = function() {
	    // Sets the lastMoved variable to the guid of the last kapp ink group moved
	    inkOnPageTrackers.push(window.NB.addObserver("annotationMovingEvent", function(event) {
	      var guid = event.eventData.guid;
	      if(lastMoved != guid && groupSet[guid]) {
	        lastMoved = guid;
	      }
	    }));

	    // Sets the lastRotated variable to the guid of the last kapp ink group rotated
	    inkOnPageTrackers.push(window.NB.addObserver("annotationRotatedEvent", function(event) {
	      var guid = event.eventData.guid;
	      if(lastRotated != guid && groupSet[guid]) {
	        lastRotated = guid;
	      }
	    }));

	    // Sets the lastResized variable to the guid of the last kapp ink group resized
	    inkOnPageTrackers.push(window.NB.addObserver("annotationResizedEvent", function(event) {
	      var guid = event.eventData.guid;
	      if(lastResized != guid && groupSet[guid]) {
	        lastResized = guid;
	      }
	    }));

	    // Checks if the deletion comes from OCR, being dropped into a widget, or being moved
	    // to a new page and tracks it accordingly
	    inkOnPageTrackers.push(window.NB.addObserver("annotationDeletedEvent", function(event) {
	      var guid = event.eventData.guid;
	      var cleanDeletion = true;
	      if(groupSet.hasOwnProperty(guid)) {
	        delete groupSet[guid];
	        if(possibleOCR) {
	          kappInkOCRDetected();
	          possibleOCR = false;
	          cleanDeletion = false;
	        }
		      if(idDroppedIntoWidget) {
		        kappInkDroppedInWidget();
		        idDroppedIntoWidget = false;
		        possibleNewPage = false;
		        cleanDeletion = false;
		      }
		      var toNewPage = false;
		      if(possibleNewPage && possibleNewPage == guid) {
		        kappInkMovedNewPage();
		        toNewPage = true;
		        cleanDeletion = false;
		      }
		      if(cleanDeletion) {
		        kappInkDeleted();
		      }
		    }
	    }));

	    // Primes the possibleOCR variable if text is inserted
	    // Adds the newly created lines to lineSet if created from a
	    // move to a new page
	    inkOnPageTrackers.push(window.NB.addObserver("annotationAddedEvent", function(event) {
	      var guid = event.eventData.guid;
	      var page = NB.document.getPage(event.eventData.pageId);
	      var annotation = page.getObjects()[guid];
	      if(annotation) {
	        if(annotation.type == "text") {
	          possibleOCR = true;
	          setTimeout(function() {
	            possibleOCR = false;
	          }, 1000);
	        } else if(annotation.type == "group" && possibleNewPage) {
	          groupSet[guid] = true;
	        }
	      }
	    }));

	    // Checks if a kapp ink group has been rotated or resized since selecting
	    inkOnPageTrackers.push(window.NB.addObserver("annotationDeSelectedEvent", function(event) {
	      if(lastResized) {
	        kappInkResized();
	        lastResized = false;
	      }
	      if(lastRotated) {
	        kappInkRotated();
	        lastRotated = false;
	      }
	    }));

	    // Checks for color and thickness changes
	    inkOnPageTrackers.push(window.NB.addObserver("propertyChangedEvent", function(event) {
	      var guid = Object.keys(event.eventData)[2];
	      var groupId = event.eventData.groupGuid;
	      if(groupSet.hasOwnProperty(groupId)) {
	        var thisChange = event.eventData[guid];
	        if(groupId != propertyChangedGroup || !propertyChangeEquality(thisChange, propertyChangedChange)) {
	          propertyChanged(thisChange);
	          propertyChangedChange = thisChange;
	          propertyChangedGroup = groupId;
	        }
	      }
	    }));

	    // If kapp ink was partially erased, finds and adds the new group
	    // Checks if ink was drawn on kapp ink
	    inkOnPageTrackers.push(window.NB.addObserver("inkingFinishedEvent", function(event) {
	      possibleOCR = false;
	      var objects = NB.page.getObjects();
	      if(objects) {
	        var groups = Object.keys(groupSet);
	        for(var i = 0; i < groups.length; i++) {
	          if(!objects[groups[i]]) {
	            kappInkErased();
	            delete groupSet[groups[i]];
	            groupSet[event.eventData.guid] = true;
	            break;
	          }
	        }
	      }
	    }));

	    // Checks if kapp ink was ungrouped
	    inkOnPageTrackers.push(window.NB.addObserver("annotationsUngroupedEvent", function(event) {
	      var groupID = event.eventData.ungroupedGroup;
	      if(groupSet[groupID]) {
	        kappDrawingUngrouped();
	        delete groupSet[groupID];
	      }
	    }));

	    // Primes possibleNewPage with the guid of the moving object
	    inkOnPageTrackers.push(window.NB.addObserver("dndStartEvent", function(event) {
	      var guid = Object.keys(event.eventData)[0];
	      if(groupSet.hasOwnProperty(guid)) {
	        possibleNewPage = guid;
	      }
	    }));

	    // Checks if a kapp ink object has been moved
	    // Checks if drop event was on top of a widget and primes idDroppedIntoWidget if so
	    inkOnPageTrackers.push(window.NB.addObserver("dndDropEvent", function(event) {
	      if(lastMoved) {
	        kappInkMoved();
	        lastMoved = false;
	      }
	      if(possibleNewPage) {
	        setTimeout(function() {
	          possibleNewPage = false;
	        }, 500);
	      }
	      var guid = Object.keys(event.eventData)[0];
	      if(groupSet.hasOwnProperty(guid)) {
	        var location = event.eventData[Object.keys(event.eventData)[0]];
	        var widget = findWidgetAtPoint(location.x, location.y);
	        if(widget) {
	          idDroppedIntoWidget = guid;
	          setTimeout(function() {
	            if(idDroppedIntoWidget) {
	              idDroppedIntoWidget = false;
	            }
	          }, 1000);
	        }
	      }
	    }));
	};

	Analytics.prototype.disableInkOnPageTracking = function() {
	    for(var i = 0; i < inkOnPageTrackers.length; i++) {
	      NB.removeObserver(inkOnPageTrackers[i]);
	    }
	    inkOnPageTrackers = [];
	};

	// Variables to store current and previous states to detect behaviors
	// Notebook can't natively track
	var lastRotated = false;
	var lastMoved = false;
	var lastResized = false;
	var possibleOCR = false;
	var groupViewed = false;
	var propertyChangedGroup = 0;
	var propertyChangedChange = {};
	var possibleNewPage = false;
	var idDroppedIntoWidget = false;

	// ---------------------------------------------------------------------------------
	// Functions called when our ink from the kapp board is manipulated within Notebook |
	// ---------------------------------------------------------------------------------

	function kappInkDeleted() {
	  that.sendTrackedEvent('ink', 'manipulate', 'delete', 1);
	}

	function kappInkMoved() {
	  that.sendTrackedEvent('ink', 'manipulate', 'move', 1);
	}

	function kappInkResized() {
	  that.sendTrackedEvent('ink', 'manipulate', 'resize', 1);
	}

	function kappInkRotated() {
	  that.sendTrackedEvent('ink', 'manipulate', 'rotate', 1);
	}

	function kappInkColorChange(color) {
	  that.sendTrackedEvent('ink', 'manipulate', 'color', 1);
	}

	function kappInkThicknessChange(thickness) {
	  that.sendTrackedEvent('ink', 'manipulate', 'thickness', 1);
	}

	function kappInkDrawnOver() {
	  that.sendTrackedEvent('ink', 'manipulate', 'ink on top', 1);
	}

	function kappInkErased() {
	  that.sendTrackedEvent('ink', 'manipulate', 'erase', 1);
	}

	function kappDrawingUngrouped() {
	  that.sendTrackedEvent('ink', 'manipulate', 'ungroup', 1);
	}

	function kappInkMovedNewPage() {
	  that.sendTrackedEvent('ink', 'manipulate', 'move to new page', 1);
	}

	function kappInkDroppedInWidget() {
	  that.sendTrackedEvent('ink', 'manipulate', 'dropped into widget', 1);
	}

	function kappInkOCRDetected() {
	  that.sendTrackedEvent('ink', 'manipulate', 'OCR performed', 1);
	}

	// Checks whether the thickness and color of two objects are the same
	function propertyChangeEquality(a, b) {
	  var colorEqual = true;
	  if(a.color || b.color) {
	    if(a.color && b.color) {
	      colorEqual = a.color[0] == b.color[0] &&
	        a.color[1] == b.color[1] && a.color[2] == b.color[2];
	    } else {
	      colorEqual = false;
	    }
	  }
	  return(a.thickness == b.thickness && colorEqual);
	}

	// Determine whether color or thickness was changed and call the
	// approrpiate tracking function
	function propertyChanged(property) {
	  if(property.color) {
	    kappInkColorChange(property.color);
	  } else if(property.thickness) {
	    kappInkThicknessChange(property.thickness);
	  }
	}

	// Determine if there is a widget at a point on the NB page
	function findWidgetAtPoint(x, y) {
	  var widgets = NB.page.getObjectsByType("widget");
	  var objects = NB.page.getObjects();
	  var goodWidget = false;
	  var length = widgets ? widgets.length : 0;
	  for(var i = 0; i < length; i++) {
	    var widget = objects[widgets[i]];
	    if(widget.contains(x, y)) {
	      return widget;
	    }
	  }
	}

	return Analytics;

}();