// Handles messaging to and from the dialog and fallback ink grouping widget. Connects
// the dialog box and the SessionHandler.

var NBMessenger = function() {

	var dialog;
	var dialogMessageCount;

	var widget;
	
	// Used to disable ink tracking while the dialog draws the ink. Not ideal to do this,
	// but unless we do, drawing kapp board contents is unusably slow with large amounts
	// of ink.
	var analytics;
	
	var colorData;

	function NBMessenger(newAnalytics) {
		analytics = newAnalytics;
		dialogMessageCount = -1;
		colorData = [];
	}

	// Creates the dialog window
	NBMessenger.prototype.openDialog = function(state, onOpenCallback, onCloseCallback) {
		if (dialog) return;

		// Disable page ink tracking until the addon has finished inking
		// Otherwise, it's unusably slow with large numbers of messages
		if(analytics) {
			analytics.disableInkOnPageTracking();
		}

		// Add saved colordata from time based color to state before
		// sending to the dialog
		state.colorData = colorData;

		var dialogCloseCleanup = function(inkGrouped) {
			// Clear observers
			clearObservers(observers);
	  		// Perform cleanup
			closeDialog(inkGrouped);
			onCloseCallback();
			NB.undomanager.stopRecord();
		};

		var observers = initializeObservers(state, dialogCloseCleanup, onOpenCallback);

		var data = {};
		data.index = "dialog/index.html";
		data.title = NB.localize("SMART kapp Board View", "dialog title");
		setDialogSize(state, data);		
		dialog = NB.ui.dialog.open(data);
		dialog.confirmOnClose(function() {
			dialogCloseCleanup();
			return false;
		});
		printDebug("Dialog opened:");
	};

	// Clear the dialog and saved color data
	NBMessenger.prototype.clearDialog = function(state) {
		colorData = [];
		var payload = constructInitialData(state);
		messageDialog("clear", payload);
	};

	// Update the dialog with new messages
	NBMessenger.prototype.updateDialog = function(data) {
		if(dialog && dialogMessageCount >= 0) {
			var payload = constructUpdateData(data);
			dialog.send("update", payload);
			dialogMessageCount = data.messageCount;
		}
	};

	// Let the dialog know it is no longer connected to the board
	NBMessenger.prototype.disconnectDialog = function() {
		messageDialog("disconnect", "");
	};

	NBMessenger.prototype.clearSessionData = function() {
		colorData = [];
	};

	// Sets the size of the dialog window to a maximum of 900 x 753. Determines the
	// aspect ratio based upon the kapp board dimensions, and the size based on the
	// available area in Notebook
	function setDialogSize(state, data) {
		var pageSize = NB.page.getVisibleArea();
		// Available width and height for canvas in dialog
		// Dialog has a 1024 x 768 maximum size, 112px of controls to the side, and 70px of controls below
		var totalWidth = Math.min(900, Math.max(800, pageSize.width * NB.page.zoom));
		var totalHeight =  Math.min(753, Math.max(600, pageSize.height * NB.page.zoom - 35));
		var controlsWidth = totalWidth * 1/9;
		var controlsHeight = totalHeight * 1/10;
		var availableWidth = totalWidth * 8/9;
		var availableHeight = totalHeight * 9/10;
		var aspect = state.session.boardWidth / state.session.boardHeight;
		if(aspect > (availableWidth / availableHeight)) {
			availableHeight = availableWidth / aspect;
		} else {
			availableWidth = availableHeight * aspect;
		}
		
		// Padding back out for control spaces
		data.width = availableWidth + controlsWidth;
		data.height = availableHeight + controlsHeight;
	}

	// Initializes the widget and dialog's observers
	function initializeObservers(state, dialogCloseCleanup, onDialogOpenCallback) {
		var observers = [];

		// When the Dialog is loaded and ready for initialization
		observers.push(window.NB.addObserver("wave", function() {
			printDebug("Dialog: Wave");
			onDialogOpenCallback();
		    var payload = constructInitialData(state);
		    dialog.send("initialize", payload);
		}));
		
		// When the Dialog fails the connection to the server,
		// use the data from the add-on instead
		observers.push(window.NB.addObserver("getMessagesFailed", function(event) {
			printDebug("Dialog: getMessagesFailed");
			dialog.send("fallback", state);
		}));

		// Called after the dialog is finished drawing. Makes sure the
		// fallback widget doesn't double group anything.
		observers.push(window.NB.addObserver("dialogFinishedDraw", function(event) {
			printDebug("Dialog: finishedDraw");
			dialogCloseCleanup(true);
		}));

		// When the Dialog has finished loading messages
		observers.push(window.NB.addObserver("loadedWave", function(event) {
			printDebug("Dialog: loadedWave");
			dialogMessageCount = event.eventData.id;
		}));

		// When the Dialog wants to draw to a new page
		observers.push(window.NB.addObserver("dialogStartingDraw", function(event) {
			printDebug("Dialog: startingDraw");
			NB.undomanager.startRecord(NB.undomanager.undotype.eWriting);
			var prototype = NB.objectPrototype.file("inkGrouper.galleryitem");
			widget = NB.addObject(prototype);
		}));

		observers.push(window.NB.addObserver("widgetWave", function(event) {
			printDebug("Widget: wave");
			dialog.send("readyToDraw", "");
		}));

		// When the Dialog has color updates for the add-on
		observers.push(window.NB.addObserver("updateColors", function(event) {
			printDebug("Dialog: updateColors");
			var indices = event.eventData.indices;
			var color = event.eventData.color;
			for(var i = 0; i < indices.length; i++) {
				colorData[indices[i]] = color;
			}
		}));

		// Adding the newly created ink group for tracking purposes
		observers.push(window.NB.addObserver("addNewGroupId", function(event) {
		  printDebug("Widget: newGroup");
		  if(analytics) {
		  	analytics.addInkGroupToTracking(event.eventData);
		  }
		}));

		return observers;
	}

	// Clears all observers in the passed in array
	function clearObservers(observers) {
		for(var i = 0; i < observers.length; i++) {
			NB.removeObserver(observers[i]);
		}
	}

	// Checks if the dialog exists, and sends data to it if it does
	function messageDialog(message, data) {
		if(dialog) {
			dialog.send(message, data);
		}
	}

	// Sends dialog the initial block of data (everything except messages)
	function initDialog() {
	  if (dialog) {
	    var payload = constructInitialData(data);
	    dialog.send("initialize", payload);
	  }
	}

	// Closes the dialog, groups ink using the fallback grouping widget if necessary
	function closeDialog(inkGrouped) {
	  if (dialog) {
	    dialog.close();
	  }
	  dialog = undefined;
	  if(inkGrouped) {
	    widget.deleteObject();
	    widget = undefined;
	  } else if(widget) {
	    var guid = widget.guid;
	    var eventData = {};
	    eventData[guid] = "";
	    setTimeout(function() {
	      NB.postEvent("groupInkAndClose", eventData);
	      widget = undefined;
	    }, 250);
	  }
	  // Enabling ink tracking now that drawing is done
	  if(analytics) {
	  	analytics.enableInkOnPageTracking();
	  }
	}

	// Creates a copy of header information used to initialize the Dialog
	function constructInitialData(data) {
	  var dup = {};
	  var keys = ["messages"];
	  var keySet = Object.keys(data);
	  for (var i = 0; i < keySet.length; i++) {
	    if(keys.indexOf(keySet[i]) == -1) {
	      dup[keySet[i]] = data[keySet[i]];
	    }
	  }
	  return dup;
	}

	// Creates a deep copy of Messages, Keyframes, Snapshots, and status
	// to send over to update the Dialog
	function constructUpdateData(data) {
	  var copy = {};
	  copy.messages = [];
	  for(var i = dialogMessageCount; i < data.messageCount; i++) {
	    copy.messages.push(data.messages[i]);
	  }
	  copy.keyframes = data.keyframes;
	  copy.keyframeCount = data.keyframeCount;
	  copy.snapshots = data.snapshots;
	  copy.snapshotCount = data.snapshotCount;
	  copy.status = data.status;
	  return copy;
	}

	return NBMessenger;

}();